@@ -8,4 +8,4 @@ lang = 'en'
 if 'CUSTOM_LANGUAGE' in os.environ:
     lang = os.environ['CUSTOM_LANGUAGE']
 
-ms = Translator(Path(__file__).parent, 'en', lang)
\ No newline at end of file
+ms = Translator(Path(__file__).parent, lang)
\ No newline at end of file
