@@ -1,19 +1,14 @@
-import time
-from datetime import datetime
 import os
-import glob
 from pathlib import Path
-import csv
 from urllib.parse import urlparse
-import subprocess
 import string 
 import random
 import math
-import base64
-import os
+import re
+import warnings
+from unidecode import unidecode
+from functools import wraps
 
-import ipyvuetify as v
-import pandas as pd
 import ee
 from cryptography.fernet import Fernet
 
@@ -47,50 +42,7 @@ def show_component(widget):
     elif 'd-none' in str(widget.class_):
         widget.class_ = widget.class_.replace('d-none', '')
         
-    return 
-
-def get_gaul_dic():
-    """
-    Create the list of the country code in the FAO GAUL norm using the CSV file provided in utils
-        
-    Return:
-        (dict): the countries FAO_GAUL codes labelled with english country names
-    """
-    
-    # file path
-    path = Path(__file__).parent.joinpath('country_code.csv')
-    
-    # get the df and sort by country name
-    df = pd.read_csv(path).sort_values(by=['ADM0_NAME'])
-    
-    # create the dict
-    fao_gaul = {row['ADM0_NAME'] : row['ADM0_CODE'] for i, row in df.iterrows()}
-        
-    return fao_gaul
-
-def get_iso_3(adm0):
-    """
-    Get the iso_3 code of a country_selection. Uses the fips_code if the iso-3 is not available
-    
-    Args:
-        adm0 (int): the country adm0 code FAO GAUL 2015
-        
-    Return:
-        (str): the 3 letters of the iso_3 country code
-    """
-    
-    # file path
-    path = Path(__file__).parent.joinpath('country_code.csv')
-    
-    # get the df
-    df = pd.read_csv(path)
-    
-    row = df[df['ADM0_CODE'] == adm0]
-    code = None
-    if len(row):
-        code = row['ISO 3166-1 alpha-3'].values[0]
-        
-    return code
+    return
     
 def create_download_link(pathname):
     """
@@ -119,15 +71,15 @@ def create_download_link(pathname):
 
 def is_absolute(url):
     """
-    Check if the given url is an absolute or relative path
+    Check if the given URL is an absolute or relative path
     
     Args:
-        url (str): the url to test
+        url (str): the URL to test
         
     Return:
         (bool): True if absolute else False
     """
-    return bool(urlparse(url).netloc)
+    return bool(urlparse(str(url)).netloc)
 
 def random_string(string_length=3):
     """
@@ -172,7 +124,7 @@ def init_ee():
     """
     Initialize earth engine according to the environment. 
     It will use the creddential file if the EE_PRIVATE_KEY env variable exist. 
-    Otherwise it use the simple Initilize command (asking the user to register if necessary)
+    Otherwise it use the simple Initialize command (asking the user to register if necessary)
     """
     
     # only do the initialization if the credential are missing
@@ -188,7 +140,7 @@ def init_ee():
             fernet = Fernet(key)
             
             # decrypt the key
-            json_encrypted = Path(__file__).parent.joinpath('encrypted_key.json')
+            json_encrypted = Path(__file__).parent/'encrypted_key.json'
             with json_encrypted.open('rb') as f:
                 json_decripted = fernet.decrypt(f.read()).decode()
                 
@@ -204,5 +156,114 @@ def init_ee():
         # if in local env use the local user credential
         else:
             ee.Initialize()
+        
+    return
+
+def catch_errors(alert, debug=False):
+    """
+    Decorator to execute try/except sentence
+    and catch errors in the alert message.
+    If debug is True then the error is raised anyway
+    
+    Params:
+        alert (sw.Alert): Alert to display errors
+        debug (bool): Wether to raise the error or not, default to false
+    """
+    def decorator_alert_error(func):
+        @wraps(func)
+        def wrapper_alert_error(*args, **kwargs):
+            try:
+                value = func(*args, **kwargs)
+            except Exception as e:
+                alert.add_msg(f'{e}', type_='error')
+                if debug:
+                    raise e
+            return value
+        return wrapper_alert_error
+    return decorator_alert_error
+
+def need_ee(func):
+    """
+    Decorator to execute check if the object require EE binding.
+    Trigger an exception if the connection is not possible. 
+    
+    Params:
+        func (obj): the object on which the decorator is applied
+    """
+    @wraps(func)
+    def wrapper_ee(*args, **kwargs):
+        
+        # try to connect to ee 
+        try: 
+            init_ee()
+        except Exception as e:
+            raise Exception ('This function needs an Earth Engine authentication')
             
-    return
\ No newline at end of file
+        return func(*args, **kwargs)
+        
+    return wrapper_ee
+
+def loading_button(alert=None, button=None, debug=False):
+    """
+    Decorator to execute try/except sentence and toggle loading button object.
+    Designed to work within the Tile object, or any object that have a self.btn and self.alert set.
+    
+    Params:
+        button (sw.Btn, optional): Toggled button
+        alert (sw.Alert, optional): the alert to display the error message
+        debug (bool, optional): wether or not the exception should stop the execution. default to False
+    """
+    
+    def decorator_loading(func):
+        
+        @wraps(func)
+        def wrapper_loading(self, *args, **kwargs):
+            
+            # set btn and alert
+            # Change name of variable to assign it again in this scope
+            button_ = self.btn if not button else button
+            alert_ = self.alert if not alert else alert
+
+            button_.toggle_loading() # Start loading 
+            value = None
+            try:
+                # Catch warnings in the process function
+                with warnings.catch_warnings(record=True) as w:
+                    value = func(self, *args, **kwargs)
+                
+                # Check if there are warnings in the function and append them
+                # Use append msg due to several warnings could be triggered
+                if w: [
+                    alert_.append_msg(warning.message.args[0], type_='warning') 
+                    for warning in w
+                ]
+                  
+            except Exception as e:
+                alert_.add_msg(f'{e}', 'error')
+                if debug:
+                    button_.toggle_loading() # Stop loading button if there is an error
+                    raise e
+                    
+            button_.toggle_loading() # Stop loading button
+            
+            return value
+        return wrapper_loading
+    return decorator_loading
+
+
+def normalize_str(msg, folder=True):
+    """
+    Normalize an str to make it compatible with file naming (no spaces, special chars ...etc)
+    
+    Params:
+        msg (str): the string to sanitise
+        folder (optional|bool): if the name will be used for folder naming or for display. if display, <'> and < > characters will be kept 
+        
+    Return:
+        (str): the modified str
+    """
+    
+    regex = '[^a-zA-Z\d\-\_]' if folder else '[^a-zA-Z\d\-\_\ \']'
+    
+    return re.sub(regex, '_', unidecode(msg))
+    
\ No newline at end of file
