@@ -66,6 +66,18 @@ class SepalWidget(v.VuetifyWidget):
         
         return self
     
+    def reset(self):
+        """
+        Clear the widget v_model. Need to be extented in custom widgets to fit the structure of the actual input
+        
+        Return:
+            self
+        """
+        
+        self.v_model = None
+        
+        return self
+    
 class Markdown(v.Layout, SepalWidget):
     """
     Custom Layout based on the markdown text given
@@ -100,14 +112,13 @@ class Tooltip(v.Tooltip):
     Custom widget to display tooltip when mouse is over widget
 
     Args:
-        widget (DOM.widget): widget used to display tooltip
+        widget (Vuetify.widget): widget used to display tooltip
         tooltip (str): the text to display in the tooltip            
     """
     
     def __init__(self, widget, tooltip, *args, **kwargs):
         
-        
-        self.bottom=True
+
         self.v_slots=[{
             'name': 'activator',
             'variable': 'tooltip',
